@@ -1,12 +1,12 @@
 """Contains various object definitions needed by the weather utility."""
 
 weather_copyright = """\
-# Copyright (c) 2006-2020 Jeremy Stanley <fungi@yuggoth.org>. Permission to
+# Copyright (c) 2006-2023 Jeremy Stanley <fungi@yuggoth.org>. Permission to
 # use, copy, modify, and distribute this software is granted under terms
 # provided in the LICENSE file distributed with this software.
 #"""
 
-weather_version = "2.4.1"
+weather_version = "2.4.4"
 
 radian_to_km = 6372.795484
 radian_to_mi = 3959.871528
@@ -95,7 +95,19 @@ class Selections:
         return None
     def get_bool(self, option, argument=None):
         """Get data and coerce to a boolean if necessary."""
-        return bool(self.get(option, argument))
+        # Mimic configparser's getboolean() method by treating
+        # false/no/off/0 as False and true/yes/on/1 as True values,
+        # case-insensitively
+        value = self.get(option, argument)
+        if isinstance(value, bool):
+            return value
+        if isinstance(value, str):
+            vlower = value.lower()
+            if vlower in ('false', 'no', 'off', '0'):
+                return False
+            elif vlower in ('true', 'yes', 'on', '1'):
+                return True
+        raise ValueError("Not a boolean: %s" % value)
     def getint(self, option, argument=None):
         """Get data and coerce to an integer if necessary."""
         value = self.get(option, argument)
@@ -221,18 +233,10 @@ def get_uri(
             data = urlopen(uri).read().decode("utf-8")
         except URLError:
             if ignore_fail: return ""
-            else:
-                import os, sys, traceback
-                message = "%s error: failed to retrieve\n   %s\n   %s" % (
-                        os.path.basename( sys.argv[0] ),
-                        uri,
-                        traceback.format_exception_only(
-                            sys.exc_type,
-                            sys.exc_value
-                        )[0]
-                    )
-                sys.stderr.write(message)
-                sys.exit(1)
+            import os, sys
+            sys.stderr.write("%s error: failed to retrieve\n   %s\n\n" % (
+                os.path.basename( sys.argv[0] ), uri))
+            raise
         # Some data sources are HTML with the plain text wrapped in pre tags
         if "<pre>" in data:
             data = data[data.find("<pre>")+5:data.find("</pre>")]
@@ -373,7 +377,7 @@ def get_options(config):
 
     # the -a/--alert option
     if config.has_option("default", "alert"):
-        default_alert = bool(config.get("default", "alert"))
+        default_alert = config.getboolean("default", "alert")
     else: default_alert = False
     option_parser.add_option("-a", "--alert",
         dest="alert",
@@ -428,7 +432,7 @@ def get_options(config):
 
     # the -f/--forecast option
     if config.has_option("default", "forecast"):
-        default_forecast = bool(config.get("default", "forecast"))
+        default_forecast = config.getboolean("default", "forecast")
     else: default_forecast = False
     option_parser.add_option("-f", "--forecast",
         dest="forecast",
@@ -456,7 +460,7 @@ def get_options(config):
 
     # the --imperial option
     if config.has_option("default", "imperial"):
-        default_imperial = bool(config.get("default", "imperial"))
+        default_imperial = config.getboolean("default", "imperial")
     else: default_imperial = False
     option_parser.add_option("--imperial",
         dest="imperial",
@@ -487,7 +491,7 @@ def get_options(config):
 
     # the -m/--metric option
     if config.has_option("default", "metric"):
-        default_metric = bool(config.get("default", "metric"))
+        default_metric = config.getboolean("default", "metric")
     else: default_metric = False
     option_parser.add_option("-m", "--metric",
         dest="metric",
@@ -497,7 +501,7 @@ def get_options(config):
 
     # the -n/--no-conditions option
     if config.has_option("default", "conditions"):
-        default_conditions = bool(config.get("default", "conditions"))
+        default_conditions = config.getboolean("default", "conditions")
     else: default_conditions = True
     option_parser.add_option("-n", "--no-conditions",
         dest="conditions",
@@ -507,7 +511,7 @@ def get_options(config):
 
     # the --no-cache option
     if config.has_option("default", "cache"):
-        default_cache = bool(config.get("default", "cache"))
+        default_cache = config.getboolean("default", "cache")
     else: default_cache = True
     option_parser.add_option("--no-cache",
         dest="cache",
@@ -517,7 +521,7 @@ def get_options(config):
 
     # the --no-cache-data option
     if config.has_option("default", "cache_data"):
-        default_cache_data = bool(config.get("default", "cache_data"))
+        default_cache_data = config.getboolean("default", "cache_data")
     else: default_cache_data = True
     option_parser.add_option("--no-cache-data",
         dest="cache_data",
@@ -527,7 +531,7 @@ def get_options(config):
 
     # the --no-cache-search option
     if config.has_option("default", "cache_search"):
-        default_cache_search = bool(config.get("default", "cache_search"))
+        default_cache_search = config.getboolean("default", "cache_search")
     else: default_cache_search = True
     option_parser.add_option("--no-cache-search",
         dest="cache_search",
@@ -537,7 +541,7 @@ def get_options(config):
 
     # the -q/--quiet option
     if config.has_option("default", "quiet"):
-        default_quiet = bool(config.get("default", "quiet"))
+        default_quiet = config.getboolean("default", "quiet")
     else: default_quiet = False
     option_parser.add_option("-q", "--quiet",
         dest="quiet",
@@ -556,7 +560,7 @@ def get_options(config):
 
     # the -v/--verbose option
     if config.has_option("default", "verbose"):
-        default_verbose = bool(config.get("default", "verbose"))
+        default_verbose = config.getboolean("default", "verbose")
     else: default_verbose = False
     option_parser.add_option("-v", "--verbose",
         dest="verbose",
@@ -604,7 +608,11 @@ def get_config():
         "weatherrc"
         ]
     for rcfile in rcfiles:
-        if os.access(rcfile, os.R_OK): config.read(rcfile)
+        if os.access(rcfile, os.R_OK):
+            if pyversion("3"):
+                config.read(rcfile, encoding="utf-8")
+            else:
+                config.read(rcfile)
     for section in config.sections():
         if section != section.lower():
             if config.has_section(section.lower()):
@@ -640,7 +648,10 @@ def integrate_search_cache(config, cachedir, setpath):
             pass
         return config
     scache = configparser.ConfigParser()
-    scache.read(scache_fn)
+    if pyversion("3"):
+        scache.read(scache_fn, encoding="utf-8")
+    else:
+        scache.read(scache_fn)
     for section in scache.sections():
         if not config.has_section(section):
             config.add_section(section)
@@ -738,7 +749,10 @@ def guess(
                     gzip.open(datafile).read().decode("utf-8") )
             else: stations.readfp( gzip.open(datafile) )
         else:
-            stations.read(datafile)
+            if pyversion("3"):
+                stations.read(datafile, encoding="utf-8")
+            else:
+                stations.read(datafile)
     else:
         message = "%s error: can't find \"%s\" data file\n" % (
             os.path.basename( sys.argv[0] ),
@@ -756,7 +770,10 @@ def guess(
                 zones.read_string( gzip.open(datafile).read().decode("utf-8") )
             else: zones.readfp( gzip.open(datafile) )
         else:
-            zones.read(datafile)
+            if pyversion("3"):
+                zones.read(datafile, encoding="utf-8")
+            else:
+                zones.read(datafile)
     else:
         message = "%s error: can't find \"%s\" data file\n" % (
             os.path.basename( sys.argv[0] ),
@@ -783,7 +800,10 @@ def guess(
                         gzip.open(datafile).read().decode("utf-8") )
                 else: airports.readfp( gzip.open(datafile) )
             else:
-                airports.read(datafile)
+                if pyversion("3"):
+                    airports.read(datafile, encoding="utf-8")
+                else:
+                    airports.read(datafile)
         else:
             message = "%s error: can't find \"%s\" data file\n" % (
                 os.path.basename( sys.argv[0] ),
@@ -872,7 +892,10 @@ def guess(
                         gzip.open(datafile).read().decode("utf-8") )
                 else: zctas.readfp( gzip.open(datafile) )
             else:
-                zctas.read(datafile)
+                if pyversion("3"):
+                    zctas.read(datafile, encoding="utf-8")
+                else:
+                    zctas.read(datafile)
         else:
             message = "%s error: can't find \"%s\" data file\n" % (
                 os.path.basename( sys.argv[0] ),
@@ -930,7 +953,10 @@ def guess(
                         gzip.open(datafile).read().decode("utf-8") )
                 else: places.readfp( gzip.open(datafile) )
             else:
-                places.read(datafile)
+                if pyversion("3"):
+                    places.read(datafile, encoding="utf-8")
+                else:
+                    places.read(datafile)
         else:
             message = "%s error: can't find \"%s\" data file\n" % (
                 os.path.basename( sys.argv[0] ),
@@ -1147,7 +1173,10 @@ def guess(
             )
         try:
             scache_existing = configparser.ConfigParser()
-            scache_existing.read(scache_fn)
+            if pyversion("3"):
+                scache_existing.read(scache_fn, encoding="utf-8")
+            else:
+                scache_existing.read(scache_fn)
             if not scache_existing.has_section(search[0]):
                 scache_fd = codecs.open(scache_fn, "a", "utf-8")
                 scache_fd.writelines(search_cache)
@@ -1198,7 +1227,7 @@ def gecos(formatted):
     return tuple(coordinates)
 
 def correlate():
-    import codecs, csv, datetime, hashlib, os, re, sys, tarfile, time, zipfile
+    import codecs, csv, datetime, hashlib, os, re, sys, time, zipfile
     if pyversion("3"): import configparser
     else: import ConfigParser as configparser
     for filename in os.listdir("."):
@@ -1386,7 +1415,7 @@ def correlate():
     sys.stdout.write(message)
     sys.stdout.flush()
     count = 0
-    slist = codecs.open(slist_fn, "rU", "utf-8")
+    slist = codecs.open(slist_fn, "r", "utf-8")
     for line in slist:
         icao = line.split("#")[0].strip()
         if icao:
@@ -1401,7 +1430,7 @@ def correlate():
     sys.stdout.write(message)
     sys.stdout.flush()
     count = 0
-    nsdcccc = codecs.open(nsdcccc_fn, "rU", "utf-8")
+    nsdcccc = codecs.open(nsdcccc_fn, "r", "utf-8")
     for line in nsdcccc:
         line = str(line)
         fields = line.split(";")
@@ -1430,7 +1459,7 @@ def correlate():
     sys.stdout.write(message)
     sys.stdout.flush()
     count = 0
-    ourairports = open(ourairports_fn, "rU")
+    ourairports = open(ourairports_fn, "r")
     for row in csv.reader(ourairports):
         icao = row[12].lower()
         if icao in stations:
@@ -1468,7 +1497,7 @@ def correlate():
     sys.stdout.write(message)
     sys.stdout.flush()
     count = 0
-    zlist = codecs.open(zlist_fn, "rU", "utf-8")
+    zlist = codecs.open(zlist_fn, "r", "utf-8")
     for line in zlist:
         line = line.split("#")[0].strip()
         if line:
@@ -1481,7 +1510,7 @@ def correlate():
     sys.stdout.flush()
     count = 0
     cpfz = {}
-    cpfzcf = codecs.open(cpfzcf_fn, "rU", "utf-8")
+    cpfzcf = codecs.open(cpfzcf_fn, "r", "utf-8")
     for line in cpfzcf:
         fields = line.strip().split("|")
         if len(fields) == 11 \
@@ -1999,15 +2028,30 @@ def correlate():
     sys.stdout.write(message)
     sys.stdout.flush()
     airports = configparser.ConfigParser()
-    airports.read(airports_fn)
+    if pyversion("3"):
+        airports.read(airports_fn, encoding="utf-8")
+    else:
+        airports.read(airports_fn)
     places = configparser.ConfigParser()
-    places.read(places_fn)
+    if pyversion("3"):
+        places.read(places_fn, encoding="utf-8")
+    else:
+        places.read(places_fn)
     stations = configparser.ConfigParser()
-    stations.read(stations_fn)
+    if pyversion("3"):
+        stations.read(stations_fn, encoding="utf-8")
+    else:
+        stations.read(stations_fn)
     zctas = configparser.ConfigParser()
-    zctas.read(zctas_fn)
+    if pyversion("3"):
+        zctas.read(zctas_fn, encoding="utf-8")
+    else:
+        zctas.read(zctas_fn)
     zones = configparser.ConfigParser()
-    zones.read(zones_fn)
+    if pyversion("3"):
+        zones.read(zones_fn, encoding="utf-8")
+    else:
+        zones.read(zones_fn)
     qalog = []
     places_nocentroid = 0
     places_nodescription = 0
